//****************************************************************************//
//  Ukazkova knihovna exportniho pluginu pro aplikaci Katalogizace souboru    //
//****************************************************************************//
//                                                                            //
//  Verze 1.1, posledni aktualizace 7.3.2000                                  //
//  (c) Copyright 2000, Pavlik Zdenek (pavlik.zdenek@post.cz)                 //
//                                                                            //
//  Tento soubor muze byt libovolne siren. Struktura dat a nazvy              //
//  exportovanych funkci musi byt zachovany, jinak plugin nebude pro aplikaci //
//  Katalogizace souboru pouzitelny                                           //
//                                                                            //
//  Format exportu 1 :                                                        //
//  DirPath;OrigPath;Level;Descript                                           //
//       FileName;Size;Crc;DateTime;Icon;Descript                             //
//  data exportovana timto exportem mohou byt opet nactena ukazkovym importem //
//  z knihovny ImpSampl.dll                                                   //
//                                                                            //
//  Format exportu 2 :                                                        //
//  Do schranky se exportuje adresarova struktura. Nazvy adresaru jsou        //
//  odsazeny tabulatory, jejichz pocet zavisi na levelu zanoreni.             //
//                                                                            //
//****************************************************************************//

library ExpSampl;

uses
  Windows,
  SysUtils,
  MyDialog in 'MyDialog.pas';


{$R *.RES}


//***************************************************************************//
//  cast typu a konstant spolecna pro DLL knihovny                           //
//***************************************************************************//

const
  _ModuleVersion = 1;

  // styl exportu dat (tj., jak budou data dodavana procedure DoExport)
  //----------------------------------------------------------------------------
  // Zakladn styl exportu, stejn jako u DOSoveho DIR, tj. nejdrive adresare
  // a soubory z aktivni slozky a pak vypis rekurzivne pro kazdou slozku
  ExportType1 = 1;


type
  TModuleInfo = packed record
    ModuleVersion : LongWord;   // verze struktury TModuleInfo
    Description   : PChar;      // popis modulu
    ExportType    : LongWord;   // zpusob exportu dat
    Reserved1     : LongWord;   // rezervovano pro budouci pouziti
    Reserved2     : LongWord;   // rezervovano pro budouci pouziti
    // ukazatele na funkce
    Config   : Pointer;         // adresa funkce pro konfiguraci modulu
    Inic     : Pointer;         // adresa funkce pro inicializaci modulu
    DeInic   : Pointer;         // adresa funkce pro deinicializaci modulu
    DoExport : Pointer;         // adresa funkce pro export dat
  end;

  PModuleInfo = ^TModuleInfo;

  // struktura s exportovanymi daty
  TExportData = packed record
    FileName : PChar;     // jmeno souboru nebo adresare
    Descript : PChar;     // popis souboru nebo adresare
    DirPath  : PChar;     // cesta k exportovane polozce
    OrigPath : PChar;     // originalni cesta k adresari (cela cesta k adresari,
                          // odkud byla slozka puvodne nactena)
    Size     : LongWord;  // velikost souboru nebo adresare
    DateTime : LongWord;  // datum a cas soboru nebo adresare v DOSovem formatu
    Crc      : Longint;   // kontrolni soucet souboru (u adresare je CRC = -1)
    Icon     : Longint;   // cislo ikony adresare (u souboru se Icon ignoruje)
    Level    : LongWord;  // level zanoreni slozky (cislovano od 0)
  end;

  PExportData = ^TExportData;

{ Indexy ikon :
    Folder  = 0;
    Group   = 1;
    Flopy   = 2;
    Disk    = 3;
    NetDisk = 4;
    CdRom   = 5;
    RamDisk = 6;
    NetworkComputer = 7;
    EntireNetwork = 8;
    CardFile = 9;
    Recycled = 10;
    Archive  = 11;
}




//***************************************************************************//
//  definice DLL knihovny a modulu                                           //
//***************************************************************************//

function Inic1: Boolean; stdcall; forward
function DeInic1: Boolean; stdcall; forward;
function DoExport1( Data: PExportData ): Boolean; stdcall; forward;
function Inic2: Boolean; stdcall; forward
function DeInic2: Boolean; stdcall; forward;
function DoExport2( Data: PExportData ): Boolean; stdcall; forward;
function Config: Boolean; stdcall; forward


const
  // popis dynamicke knihovny
  LibraryDescription = 'RoyalSoft sample export library';

  // popis prvniho exportniho modulu
  Module1 : TModuleInfo = ( ModuleVersion : _ModuleVersion;
                            Description : 'Kompletn export vech dat';
                            ExportType  : ExportType1;
                            Reserved1: 0;
                            Reserved2: 0;
                            Config   : @Config;
                            Inic     : @Inic1;
                            DeInic   : @DeInic1;
                            DoExport : @DoExport1
                           );

  // popis druheho exportniho modulu
  Module2 : TModuleInfo = ( ModuleVersion : _ModuleVersion;
                            Description : 'Export adresov struktury do schrnky';
                            ExportType  : ExportType1;
                            Reserved1: 0;
                            Reserved2: 0;
                            Config   : @Config;
                            Inic     : @Inic2;
                            DeInic   : @DeInic2;
                            DoExport : @DoExport2
                           );

  // pomocne konstanty
  Enter = #13#10;
  Tab = #9;


var
  FileName   : String;
  DirPath    : String;
  FileHandle : Integer;
  Buffer     : array [0..2048] of Char;
//  Clipboard  : TClipboard;
  ClipboardStr : String;


//==============================================================================
// Funkce pro vraceni popisu knihovny. Musi se exportovat !!!
function GetExportLibraryDescription: PChar; stdcall;
begin
  Result := LibraryDescription;
end;


//==============================================================================
// Funkce pro vraceni struktury s informacemi o modulu s poradim Index
// Musi se exportovat !!!
function GetModuleInfo( Index: Integer): PModuleInfo; stdcall;
begin
  case Index of
    0 :  Result := @Module1;
    1 :  Result := @Module2;
    else Result := nil;
  end;
end;


//==============================================================================
// Funkce pro konfiguraci vybraneho modulu
// Nastaven nesm bt uloeno do globlnch promnnch, ale do souboru
// plugin.ini do sekce s jednoznanm nzevem pluginu.
function Config: Boolean;
begin
  Result := False;
  try
    MessageBox( nil, 'Ukzkov exportn knihovna. (c)2000 Pavlk Zdenk / RoyalSoft.',
                'Info', mb_Ok );
    Result := True;
  except
  end;
end;


//==============================================================================
// Inicializace modulu. Je volana pred zacatkem exportu. Muze byt pro kazdy
// modul jina. Navratova hodnota urcuje uspesnost inicializace.
function Inic1: Boolean; stdcall;
var
  SaveDialog : TSaveDialog;
begin
  Result := False;
  try
    // vytvoreni ukladaciho dialogu
    SaveDialog := TSaveDialog.Create;
    try
      // zobrazeni ukladaciho dialogu
      if SaveDialog.Execute then begin
        FileName := SaveDialog.FileName;
        // vytvoreni souboru
        FileHandle := FileCreate( FileName );
        // pocatecni nastaveni
        DirPath := '';
        SetCursor(LoadCursor(0,Idc_Wait));
        // navratova hodnota podle uspesnosti inicializace
        Result := (FileHandle <> -1);
      end;
    finally
      SaveDialog.Free;
    end;
  except
    MessageBox( nil, 'Dolo k chyb pi inicializaci pluginu.', 'Chyba', mb_Ok );
  end;
end;


//==============================================================================
// Deicializace modulu. Je volana po skonceni exportu. Muze byt pro kazdy
// modul ruzna. Navratova hodnota urcuje uspesnost deinicializace.
function DeInic1: Boolean; stdcall;
begin
  Result := False;
  try
    // uzavreni souboru
    FileClose( FileHandle );
    SetCursor(LoadCursor(0,Idc_Arrow));
    Result := True;
  except
    MessageBox( nil, 'Dolo k chyb pi deinicializaci pluginu.', 'Chyba', mb_Ok );
  end;
end;


//==============================================================================
// Funkce pro export dat. Je volana pro kazdou polozku. Adresar se pozna podle
// polozky CRC = -1, Icon u souboru se ignoruje.
// Navratova hodnota udava uspesnost exportu.
function DoExport1( Data: PExportData ): Boolean; stdcall;
var
  Str : String;
begin
  Result := False;
  try
    // Kontrola zmeny adresare. Pokud dojde ke zmene adresare, jsou ve
    // strukture Data ulozeny vsechny informace o tomto adresari.
    if (DirPath <> Data^.DirPath) then begin
      DirPath := Data^.DirPath;
      Str := Format( Enter+'%s;%s;%u;%s' + Enter,
                    [DirPath, Data^.OrigPath, Data^.Level, Data^.Descript] );
      StrPCopy( Buffer, Str );
      Result := -1 <>  FileWrite( FileHandle, Buffer, StrLen( Buffer ) );
    end else begin
      Str := Format( Tab+'%s;%d;%d;%d;%d;%s'+Enter,
                     [Data^.FileName, Data^.Size, Data^.Crc, Data^.DateTime,
                      Data^.Icon, Data^.Descript] );
      StrPCopy( Buffer, Str );
      Result := -1 <> FileWrite( FileHandle, Buffer, StrLen( Buffer ) );
    end;
  except
  end;
end;





//==============================================================================
// Inicializace modulu. Je volana pred zacatkem exportu. Muze byt pro kazdy
// modul jina. Navratova hodnota urcuje uspesnost inicializace.
function Inic2: Boolean; stdcall;
begin
  Result := False;
  try
    // pocatecni adresar
    DirPath := '';
    ClipboardStr := '';
    SetCursor(LoadCursor(0,Idc_Wait));
    Result := True;
  except
    MessageBox( nil, 'Doslo k chybe pri inicializaci pluginu.', 'Chyba', mb_Ok );
  end;
end;


//==============================================================================
// Deicializace modulu. Je volana po skonceni exportu. Muze byt pro kazdy
// modul ruzna. Navratova hodnota urcuje uspesnost deinicializace.
function DeInic2: Boolean; stdcall;
var
  MemHandle : LongWord;
  DestBuff  : Pointer;
  SrcBuff   : Pointer;
  ClipSize  : LongWord;
begin
  Result := False;
  try
    // Kopirovani do schranky
    if OpenClipboard( 0 ) then begin
      ClipSize := Length(ClipboardStr);
      MemHandle := GlobalAlloc( GMEM_MOVEABLE + GMEM_DDESHARE, ClipSize + 1 );
      if MemHandle <> 0 then begin
        DestBuff := GlobalLock( MemHandle );
        if DestBuff <> nil then begin
          SrcBuff := PChar(ClipboardStr);
          Move( SrcBuff^, DestBuff^, ClipSize + 1);
          SetClipboardData( CF_TEXT,  MemHandle );
          GlobalUnlock( MemHandle );
        end;
      end;
      CloseClipboard;
    end;
    SetCursor(LoadCursor(0,Idc_Arrow));
    Result := True;
  except
    MessageBox( nil, 'Doslo k chybe pri deinicializaci pluginu.', 'Chyba', mb_Ok );
  end;
end;


//==============================================================================
// Funkce pro export dat. Je volana pro kazdou polozku. Adresar se pozna podle
// polozky CRC = -1, Icon u souboru se ignoruje.
// Navratova hodnota udava uspesnost exportu.
function DoExport2( Data: PExportData ): Boolean; stdcall;
begin
  Result := False;
  try
    if (DirPath <> Data^.DirPath) then begin
      DirPath := Data^.DirPath;
      ClipboardStr := ClipboardStr + StringOfChar( Tab, Data^.Level ) +
                      Data^.FileName + Enter;
    end;
    Result := True;
  except
  end;
end;



exports
  GetModuleInfo,
  GetExportLibraryDescription;

begin
end.
