//****************************************************************************//
//  Ukazkova knihovna importniho pluginu pro aplikaci Katalogizace souboru    //
//****************************************************************************//
//                                                                            //
//  Verze 1.1, posledni aktualizace 7.3.2000                                  //
//  (c) Copyright 2000, Pavlik Zdenek (pavlik.zdenek@post.cz)                 //
//                                                                            //
//  Tento soubor muze byt libovolne siren. Struktura dat a nazvy              //
//  exportovanych funkci musi byt zachovany, jinak plugin nebude pro aplikaci //
//  Katalogizace souboru pouzitelny                                           //
//                                                                            //
//  Format importu 1 :                                                        //
//  format dat, ktery lze importovat, je stejny, jako export 1 z ukazkove     //
//  knihovny ExpSampl.dll                                                     //
//                                                                            //
//****************************************************************************//

library ImpSampl;

uses
  Windows,
  SysUtils,
  MyDialog in 'MyDialog.pas';


{$R *.RES}


//***************************************************************************//
//  cast typu a konstant spolecna pro DLL knihovny                           //
//***************************************************************************//

const
  _ModuleVersion = 1; // Nemenit !!! Udava typ struktury TModuleInfo


  // styl importu dat( tj., jak budou data dodavana procedure DoImport )
  //----------------------------------------------------------------------------
  // Zakladn styl importu. V DirPath je ulozena relativni cesta, do ktere jsou
  // ulozena data. Pro import primo do vybrane slozky je cesta prazdna.
  // Ve FileName je ulozeno jmeno souboru nebo adresare, ktery je ulozen
  // v katalogu do adresare s cestou DirPath. V OrigPath je fyzicka cesta,
  // tj. cesta k adresari na disku, ve ktere se nachazi polozka FileName.
  // Dalsi atribututy jsou velikost souboru ci adresare (Size), datum (DateTime),
  // kontrolni soucet (CRC, pro adresar musi byt CRC = -1), cislo ikony adresare
  // podle seznamu nize (Icon, u souboru se ignoruje)
  ImportType1 = 1;


type
  TModuleInfo = packed record
    ModuleVersion : LongWord;   // verze struktury TModuleInfo
    Description   : PChar;      // popis modulu
    ImportType    : LongWord;   // zpusob importu dat
    Reserved1     : LongWord;   // rezervovano pro budouci pouziti
    Reserved2     : LongWord;   // rezervovano pro budouci pouziti
    // ukazatele na funkce
    Config   : Pointer;         // adresa funkce pro konfiguraci modulu
    Inic     : Pointer;         // adresa funkce pro inicializaci modulu
    DeInic   : Pointer;         // adresa funkce pro deinicializaci modulu
    DoImport : Pointer;         // adresa funkce pro import dat
  end;

  PModuleInfo = ^TModuleInfo;

  // struktura s importovanymi daty. Vsechny polozky musi byt vyplneny.
  // Polozkam typu PChar je pridelen buffer o velikosti uvedene v zavorce.
  // Tyto buffery nejsou aplikaci Katalog meneny, proto nemusi byt napriklad
  // polozky DirPath a OrigPath zapisovany pri kazdem zavolani funkce DoImport,
  // pokud nedojde ke zmene adresare.
  TImportData = packed record
    FileName : PChar;     // jmeno souboru nebo adresare (max 256 znaku)
    Descript : PChar;     // popis souboru nebo adresare (max 2048 znaku)
    DirPath  : PChar;     // cesta importovane polozky   (max 2048 znaku)
                          // relativni cesta od polozky, do ktere se provadi import
    OrigPath : PChar;     // originalni cesta k adresari (max 2048 znaku)
                          // fyzicka cesta k adresari, odkud byla slozka nactena
    Size     : LongWord;  // velikost souboru nebo adresare
    DateTime : LongWord;  // datum a cas soboru nebo adresare v DOSovem formatu
    Crc      : Longint;   // kontrolni soucet souboru (u adresare musi byt CRC = -1)
    Icon     : Longint;   // cislo ikony adresare (u souboru se ignoruje)
  end;

  PImportData = ^TImportData;

{ Indexy ikon :
    Folder  = 0;
    Group   = 1;
    Flopy   = 2;
    Disk    = 3;
    NetDisk = 4;
    CdRom   = 5;
    RamDisk = 6;
    NetworkComputer = 7;
    EntireNetwork = 8;
    CardFile = 9;
    Recycled = 10;
    Archive  = 11;
}




//***************************************************************************//
//  definice DLL knihovny a modulu                                           //
//***************************************************************************//

function Inic1: Boolean; stdcall; forward
function DeInic1: Boolean; stdcall; forward;
function DoImport1( Data: PImportData; var Finished: Boolean ): Boolean; stdcall; forward;
function Config: Boolean; stdcall; forward


const
  // popis dynamicke knihovny
  LibraryDescription = 'RoyalSoft sample import library';

  // popis prvniho importniho modulu
  Module1 : TModuleInfo = ( ModuleVersion : _ModuleVersion;
                            Description : 'Import vech dat';
                            ImportType  : ImportType1;
                            Reserved1: 0;
                            Reserved2: 0;
                            Config   : @Config;
                            Inic     : @Inic1;
                            DeInic   : @DeInic1;
                            DoImport : @DoImport1
                           );


var
  FileName   : String;
  F : TextFile;


//==============================================================================
// Funkce pro vraceni popisu knihovny. Musi se exportovat !!!
function GetImportLibraryDescription: PChar; stdcall;
begin
  Result := LibraryDescription;
end;


//==============================================================================
// Funkce pro vraceni struktury s informacemi o modulu s poradim Index
// Musi se exportovat !!!
function GetModuleInfo( Index: Integer): PModuleInfo; stdcall;
begin
  case Index of
    0 :  Result := @Module1;
    else Result := nil;
  end;
end;


//==============================================================================
// Funkce pro konfiguraci vybraneho modulu
// Nastaven nesm bt uloeno do globlnch promnnch, ale do souboru
// plugin.ini do sekce s jednoznanm nzevem pluginu.
function Config: Boolean;
begin
  Result := False;
  try
    MessageBox( nil, 'Ukzkov importn knihovna. (c)2000 Pavlk Zdenk / RoyalSoft.',
                'Info', mb_Ok );
    Result := True;
  except
  end;
end;


//==============================================================================
// Inicializace modulu. Je volana pred zacatkem importu. Muze byt pro kazdy
// modul jina. Navratova hodnota urcuje uspesnost inicializace.
function Inic1: Boolean; stdcall;
var
  OpenDialog : TOpenDialog;
begin
  Result := False;
  try
    // vytvoreni oteviraciho dialogu
    OpenDialog := TOpenDialog.Create;
    try
      // zobrazeni oteviraciho dialogu
      if OpenDialog.Execute and (OpenDialog.FileName <> '') then begin
        FileName := OpenDialog.FileName;
        // otevreni souboru
        AssignFile( F, OpenDialog.FileName );
        Reset(F);
        // pocatecni nastaveni
        SetCursor(LoadCursor(0,Idc_Wait));
        // navratova hodnota podle uspesnosti inicializace
        Result := True;
      end;
    finally
      OpenDialog.Free;
    end;
  except
    MessageBox( nil, 'Dolo k chyb pi inicializaci pluginu.', 'Chyba', mb_Ok );
  end;
end;


//==============================================================================
// Deicializace modulu. Je volana po skonceni importu. Muze byt pro kazdy
// modul ruzna. Navratova hodnota urcuje uspesnost deinicializace.
function DeInic1: Boolean; stdcall;
begin
  Result := False;
  try
    // uzavreni souboru
    CloseFile(F);
    SetCursor(LoadCursor(0,Idc_Arrow));
    Result := True;
  except
    MessageBox( nil, 'Dolo k chyb pi deinicializaci pluginu.', 'Chyba', mb_Ok );
  end;
end;


//==============================================================================
// Funkce pro import dat. Je volana pro kazdou polozku. U adresare musi byt
// CRC = -1 a soubor se Icon ignoruje. Navratova hodnota udava uspesnost importu.
// Zakladni ikona adresare je Icon = 0. Konec importu je indikovan nastavenim
// Finish = True. Data s Finish = True nejsou jiz akceptovana.
function DoImport1( Data: PImportData; var Finished: Boolean ): Boolean; stdcall;
var
  S : String;
  P : Integer;
  I : LongWord;
  State : Boolean;
begin
  Result := False;
  Finished := False;
  try
    State := False;
    repeat
      if not Eof(F) then begin
        Readln( F, S );
        if S <> '' then begin
          // vycteni souboru nebo adresare
          if (S[1] = ' ') or (S[1] = #9) then begin
            // vycteni oddelovacu
            while (S <> '') and ((S[1] = ' ') or (S[1] = #9)) do
              S := Copy( S, 2, Length(S) - 1 );

            if S <> '' then begin
              // vycteni jmena
              P := Pos( ';', S );
              if (P = 0) or (P > 255) then Exit;
              StrPCopy( Data^.FileName, Copy( S, 1, P - 1 ) );
              S := Copy( S, P + 1, Length(S) - P );

              // vycteni velikosti
              P := Pos( ';', S );
              if (P = 0) then Exit;
              I := StrToIntDef( Copy( S, 1, P - 1 ), 0 );
              Data^.Size := I;
              S := Copy( S, P + 1, Length(S) - P );

              // vycteni CRC
              P := Pos( ';', S );
              if (P = 0) then Exit;
              I := StrToIntDef( Copy( S, 1, P - 1 ), 0 );
              Data^.Crc := I;
              S := Copy( S, P + 1, Length(S) - P );

              // vycteni datumu
              P := Pos( ';', S );
              if (P = 0) then Exit;
              I := StrToIntDef( Copy( S, 1, P - 1 ), 0 );
              Data^.DateTime := I;
              S := Copy( S, P + 1, Length(S) - P );

              // vycteni cisla ikony
              P := Pos( ';', S );
              if (P = 0) then Exit;
              I := StrToIntDef( Copy( S, 1, P - 1 ), 0 );
              Data^.Icon := I;
              S := Copy( S, P + 1, Length(S) - P );

              // vycteni popisu
              if (Length(S) < 2048) then
                StrPCopy( Data^.Descript, S );

              // indikace uspesneho nacteni polozky
              State := True
            end;

          // vypis noveho adresare
          end else begin
            // vycteni cesty
            P := Pos( ';', S );
            if (P = 0) or (P > 2048) then Exit;
            StrPCopy( Data^.DirPath, Copy( S, 1, P - 1 ) );
            S := Copy( S, P + 1, Length(S) - P );

            // vycteni puvodni cesty
            P := Pos( ';', S );
            if (P = 0) or (P > 2048) then Exit;
            StrPCopy( Data^.OrigPath, Copy( S, 1, P - 1 ) );
          end;
        end;
      end else begin
        Finished := True;
        State := True;
      end;

    until State;
    Result := True;
  except
  end;
end;



exports
  GetModuleInfo,
  GetImportLibraryDescription;


begin
end.
